\newpage

# `QEMU` introduction

## Prise en main de QEMU

### Partie 1

```bash
time qemu-system-x86_64 -cdrom lubuntu-22.04.4-desktop-amd64.iso -m 4G -smp cpus=2
```

Le temps d'exécution est de 1 minutes 45 secondes (135 secondes)

### Partie 2

```bash
time qemu-system-x86_64 -cdrom lubuntu-22.04.4-desktop-amd64.iso -m 4G -smp cpus=2 -enable-kvm
```

Le temps d'exécution est de 22 secondes

#### Gain de temps

$$
\frac{135}{22} = 7.5
$$

Le facteur de gain grâce à `KVM` est de 7.5x


#### Techniques de virtualisation

- avec `-enable-kvm`: full virtualization (hardware-assisted)

- sans `-enable-kvm`: full virtualization (binary translation)
    - Les instructions non virtualisables sont traduites en temps réel.

#### Installation de Ubuntu

Non, il n'est pas possible d'installer ubuntu car il n'y a pas de support de
stockage

\newpage

### Partie 3

#### Création de l'image `lubuntu.qcow` de 100GB au format qcow2

```bash
qemu-img create -f qcow2 lubuntu.qcow 100G
```

#### Création de l'image `disk.raw` de 100GB au format raw

```bash
qemu-img create -f raw disk.raw 100G
```

#### Questions

- En quoi l'image `qcow2` diffère-t-elle de l'image `raw` ? Décrivez en les
différences majeures.
    - Comparé à l'image `raw`, l'image `qcow2` possède un "en-tête". Dans le
    cas de l'image `raw` elle est complètement vide (remplie de 0).

- Inspectez la taille apparente (`ls -lh`) de chaque image ainsi que la taille
réelle (`ls -lhs`). Que constatez-vous ? Expliquez le comportement observez.

```bash
~/isc/virtu/disks > ls -lh
total 200K
-rw-r--r-- 1 iliya iliya 100G Mar 11 10:02 disk.raw
-rw-r--r-- 1 iliya iliya 194K Mar 11 10:02 lubuntu.qcow
```

```bash
~/isc/virtu/disks > ls -lhs
total 200K
4.0K -rw-r--r-- 1 iliya iliya 100G Mar 11 10:02 disk.raw
196K -rw-r--r-- 1 iliya iliya 194K Mar 11 10:02 lubuntu.qcow
```

Comme nous pouvons le voir ci-dessus, la sortie de ces deux commandes nous
donne un résultat peu intuitif à première vue. Nous étions censés créer deux
images de 100GB sauf que l'image `qcow2` ne fait que ~ 194KB, alors que l'image
`raw` fait bien 100GB. Sauf que dans les deux cas, `ls` nous indique que l'espace
total occupé par le répertoire courant n'est que de 200KB.

Dans le cas de la commande `ls -lhs`, le paramètre `-s` nous indique l'espace
alloué en blocs pour chaque fichier. Dans le cas de `raw` 4'000 blocs sont alloués,
quant au fichier `qcow2`, 196'000 blocs ont été alloués.

##### `qemu-img info`

La commande `qemu-img info <fichier>` permet d'obtenir des informations supplémentaires
sur les images créées.

```bash
image: lubuntu.qcow
file format: qcow2
virtual size: 100 GiB (107374182400 bytes)
disk size: 196 KiB
cluster_size: 65536
Format specific information:
    compat: 1.1
    compression type: zlib
    lazy refcounts: false
    refcount bits: 16
    corrupt: false
    extended l2: false
Child node '/file':
    filename: lubuntu.qcow
    protocol type: file
    file length: 194 KiB (198656 bytes)
    disk size: 196 KiB
```

```bash
image: disk.raw
file format: raw
virtual size: 100 GiB (107374182400 bytes)
disk size: 4 KiB
Child node '/file':
    filename: disk.raw
    protocol type: file
    file length: 100 GiB (107374182400 bytes)
    disk size: 4 KiB
```

- Quel est le contenu de `disk.raw` ?
    - Que des 0

### Partie 4

#### Création d'une VM qui utilise l'image `lubuntu.qcow` 

- Comme peut-on écrire **exactement** l'équivalent de `-hdb` avec la syntaxe
`-drive` ?
    - Lorsqu'on utilise l'argument `-drive` lors de la création de l'image,
    il est possible de spécifier une seconde fois cet argument et spécifier
    l'index de l'image `raw` de la manière suivante : `index=1`. Ceci indiquera
    que c'est l'image secondaire.

```bash
qemu-system-x86_64 -cdrom lubuntu-22.04.4-desktop-amd64.iso -m 4G -smp cpus=2 -enable-kvm -drive file=../disks/lubuntu.qcow,index=0,media=disk,format=qcow2,if=virtio
```

#### Inspection de la taille du fichier `lubuntu.qcow` avec `qemu-img info` 

Nous pouvons voir à présent que la taille du fichier fait 11.9 GiB au lieu de
196 KiB précédemment.

```bash
image: lubuntu.qcow
file format: qcow2
virtual size: 100 GiB (107374182400 bytes)
disk size: 11.9 GiB
cluster_size: 65536
Format specific information:
    compat: 1.1
    compression type: zlib
    lazy refcounts: false
    refcount bits: 16
    corrupt: false
    extended l2: false
Child node '/file':
    filename: lubuntu.qcow
    protocol type: file
    file length: 11.9 GiB (12737576960 bytes)
    disk size: 11.9 GiB
```


#### Démarrage de la VM installée

```bash
qemu-system-x86_64 -m 4G -smp cpus=2 -enable-kvm ../disks/lubuntu.qcow
```

#### Inspection du retour de la commande `lscpu` dans l'OS guest

Suite au fait que nous avions attribué 2 CPUs à la VM, `lscpu` ne détecte que
ces 2 coeurs dans l'OS guest.

En ce qui concerne le modèle du CPU, `lscpu` attribue le nom **QEMU Virtual CPU
version 2.5+**, or le nom exact du modèle du CPU de l'hôte dans mon cas est
**AMD Ryzen 7 7840HS w/ Radeon 780M Graphics**

Pour ce qui en est d'autres différences, il y a notamment le fait que dans le
cas de la VM, il n'y qu'un seul thread par coeur physique. Dans le cas du hôte,
il en possède 2 par coeur. La taille de la cache est réduite dans l'OS guest.

Finalement l'OS guest a "conscience" qu'elle est virtualisée, notamment cela
se voit clairement grâce au champs **Hypervisor vendor** (KVM) et **Virtualization
type** (full). De ce fait, les résultats de la commande `lscpu` divergent
entre l'hôte et l'OS guest.

#### Section "Virtualization features" (Hôte vs OS guest)

Comme je l'ai mentionné précédemment, dans le cas de l'OS guest, on remarque
le fait qu'il comprend qu'il est virtualisée (KVM, Full virtualization). En ce
qui concerne l'hôte, dans la partie "Virtualization features" de la commande
`lscpu`, il n'y qu'un seul champ **Virtualization** dont la valeur est AMD-V.
AMD-V est un set d'extensions matérielles sur les processeurs AMD qui permettent
la virtualisation complète est assisté (_hardware-assisted_).

#### Est-ce que l'OS guest peut être utilisée comme hyperviseur en utilisant la technique "_hardware-assisted full virtualization_" ?

Avec les arguments passés précédemment à `qemu-system-x86_64` il semblerait qu'il
ne soit pas possible d'utilisé l'OS guest comme hyperviseur avec la technique
"_hardware-assisted full virtualization_" car le CPU de l'OS guest n'a pas accès
aux instructions matérielles du CPU pour faire de la virtualisation assistée.

Cependant si l'on passe à `qemu-system-x86_64` l'option `-cpu` avec l'argument
`host`, on pourra exposer à l'OS guest toutes les fonctionnalités matérielles
(CPU) de l'hôte, notamment les instructions de virtualisation, par conséquent
il est possible grâce à cette option d'utilisé l'OS guest comme hyperviseur
avec la technique "_hardware-assisted full virtualization_".

#### L'interface réseau est-elle emulée ou paravirtualisée ?

La carte réseau de la guest OS est une `Intel Corporation 82540EM` Gigabit
Ethernet Controller. Il semblerait qu'elle soit emulée et non paravirtualisée
car l'émulation sera le comportement par défaut de `QEMU` pour un périphérique
PCI suite au fait que nous n'avons pas spécifié explicitement de paravirtualisé
ce périphérique lors de la création de la VM.

#### Où se trouve la configuration de la VM et comment est-elle définie ?

La configuration de la VM sont les paramètres passés à `qemu-system-x86_64` au
moment de sa création.

Jusqu'à présent les options pour la VM `lubuntu` sont :

```bash
qemu-system-x86_64 -m 4G -smp cpus=2 -enable-kvm lubuntu.qcow
```

où le disque `lubuntu.qcow` a été créé de la manière suivante :

```bash
qemu-img create -f qcow2 lubuntu.qcow 100G
```

#### Est-il possible de copier/coller du texte entre la VM et la machine hôte

Non.

## Optimisation des performances

### Contrôleur de disque émulé vs paravirtualisé

Nous allons à présent mesurer le temps entre le démarrage de la VM et de son
shutdown "gracieux" (à travers l'interface graphique).

| Émulé   | Paravirtualisé    |
|--------------- | --------------- |
| 1 min 59 s | 36 s   |

Le disque paravirtualisé est $\sim$ 4x plus rapide. Ces mesures ont été effectuées
en prenant la valeur du champ "total" de l'outil `time`.

### Nom du périphérique connecté à la racine `/` du FS (disque émulé)

Le nom de ce périphérique est : `/dev/sda1`

### Nom du périphérique connecté à la racine `/` du FS (disque paravirtualisé)

Le nom de ce périphérique est : `/dev/sda1`

### Investigation avec `journactl`, que trouvez-vous si vous cherchez le mot-clé `virtio` ?

La commande utilisée est : `journalctl -ka` 

Je ne trouve aucune référence à ce mot clé.

### Listez les périphériques avec la commande `lspci` 

```bash
00:00.0 Host bridge: Intel Corporation 440FX - 82441FX PMC [Natoma] (rev 02)
00:01.0 ISA bridge: Intel Corporation 82371SB PIIX3 ISA [Natoma/Triton II]
00:01.1 IDE interface: Intel Corporation 82371SB PIIX3 IDE [Natoma/Triton II]
00:01.3 Bridge: Intel Corporation 82371AB/EB/MB PIIX4 ACPI (rev 03)
00:02.3 VGA compatible controller: Device 1234:1111 (rev 02)
00:03.0 Ethernet controller: Intel Corporation 82540EM Gigabit Ethernet Controller (rev 03)
```

#### Contrôleur de stockage

Étonamment, je ne vois aucune mention d'un contrôleur de stockage.

#### Carte graphique

Je ne vois pas de carte graphique à part entière, un contrôleur VGA est simplement
mentionné.

### Redimension de le fenêtre d'affichage de `QEMU`

En redimensionnant la fenêtre, j'observe que la "vue" de la VM est entourée
de marges noires et donc l'affichage de la VM n'arrive pas à remplir la fenêtre
de `QEMU`.

### Paravirtualisation de la carte graphique

En ajoutant l'option `-vga virtio`, nous avons paravirtualisé la carte graphique.

À présent lorsqu'on redimensionne la fenêtre de `QEMU`, l'affichage de la VM
ainsi que la résolution s'adapte dynamiquement à la taille de la fenêtre. Lorsqu'on
inspecte le retour de la commande `lspci`, en ce qui concerne le champ du contrôleur
VGA, on a à présent la valeur suivante :

```bash
00:02.0 VGA compatible controller: Red Hat, Inc. Virtio GPU (rev 01)
```
