\newpage

# Docker Storage

## Exercice 1

Ajoutez/modifiez/supprimez des fichiers dans le répertoire `merged` et observez
ce qu'il se passe dans les répertoires :

- `lower`
    - Les modifications effectuées dans le répertoire `merged` ne sont pas
    reflétés dans la couche `lower`. Ceci est normal car la couche `lower` est
    immutable.

- `upper`
    -  Contient les modifications effectuées dans le répertoire `merged`.

- `merged`
    -  La couche `merged` est l'union de la couche basse avec les modifications
    dans la couche `upper`.

- Quelle est l'analogie entre le scénario que vous avez réalisé ici et les
containers `Docker` (image et couche _container_)
    - Le scénario réalisé reflète le fonctionnement des containers `Docker` où
    la couche basse (`lower`) correspond à l'image `Docker` sur laquelle est basé
    le container et la couche haute (`upper`) correspond au container instancié
    à partir d'une image.

- Que se passe-t-il réellement lorsqu'un fichier de la vue unifiée (`merged`)
est ajouté ?
    - Lorsqu'un fichier est ajouté, sa présence ne sera pas reflétée dans la
    couche basse.

- Que se passe-t-il réellement lorsqu'un fichier de la vue unifiée (`merged`)
et existant dans la couche basse est modifié ?
    - Le modifications ne seront pas propagées dans la couche basse car celle-ci
    est immutable.

- Que se passe-t-il réellement lorsqu'un fichier de la vue unifiée (`merged`)
et existant dans la couche basse est supprimé ?
    - Le modifications ne seront pas propagées dans la couche basse car celle-ci
    est immutable.

## Exercice 2

### Ajout de l'image `fedora`

```bash
docker pull fedora:41
```

### Exécution du container

```bash
docker run --rm -it fedora:41 /bin/bash
```

- Ajout d'un fichier à la racine

```bash
touch root_file
```

- Suppression de `/etc/issue` 

```bash
rm /etc/issue
```

- Ajout d'une ligne à `/etc/passwd` 

```bash
echo "Hello world" >> /etc/passwd
```

### Inspection du container

```bash
docker inspect <container_name>
```

- Combien de couches basses contient ce container et quelles sont leurs chemins
absolus dans le système de fichiers sur la machine hôte ?

```bash
docker inspect serene_banzai | jq '.[0].GraphDriver.Data'
```

Il semberait que le container contiennent 2 couches basses, leurs chemins absolu
dans la machine hôte sont délimité par un `:`

    - /var/lib/docker/overlay2/c09fee9d46e25b792b708cf644e1fae6e93b69316cacf6d601050b6947c2e671-init/diff
    - /var/lib/docker/overlay2/4760625ee9a5f5e1b9b35b91cc070433b30ba2a931a70b770dafc1b0153e3952/diff

- Quel est le chemin absolu de la couche container _writable_ ?

```bash
"UpperDir": "/var/lib/docker/overlay2/c09fee9d46e25b792b708cf644e1fae6e93b69316cacf6d601050b6947c2e671/diff"
```

- À l'aide de la commande `tree`, listez le contenu complet de la couche container
_writable_ :

```bash
sudo tree /var/lib/docker/overlay2/c09fee9d46e25b792b708cf644e1fae6e93b69316cacf6d601050b6947c2e671/diff
```

```bash
├── etc
│   ├── issue
│   └── passwd
└── root_file
```

- Sur la machine hôte, quel est le chemin absolu du répertoire correspondant
au _rootfs_ du container ?

```bash
docker inspect serene_banzai | jq '.[0].GraphDriver.Data.MergedDir'
```

- /var/lib/docker/overlay2/c09fee9d46e25b792b708cf644e1fae6e93b69316cacf6d601050b6947c2e671/merged

Déplacez-vous dans le répertoire correspondant au _rootfs_ du container, puis
créez le fichier `Hello_from_the_outside`.

```bash
sudo su
cd /var/lib/docker/overlay2/c09fee9d46e25b792b708cf644e1fae6e93b69316cacf6d601050b6947c2e671/merged
touch Hello_from_the_outside
```

- Suite à cette opération, qu'observez vous dans le container lorsque vous listez
le contenu du répertoire racine ?
    - Quand on retourne inspecter les fichiers à la racine du container, le fichier
    `Hello_from_the_outside` créé depuis la machine hôte, est bien dans le container.


- Est-ce qu'il possible de simplement utiliser la commande mount sur le serveur
pour obtenir les mêmes informations (à la place d'utiliser la commande 
`docker inspect`) ? Justifiez votre démarche.
    - Je suppose que non car la commande `mount` en elle-même ne fourni pas
    d'informations sur le métadonnées d'un container et donc ne permet pas de
    de connaître le chemin exact vers les diverses couches du container.


## Exercice 3

Dans cette exercice on désire modifier un container existant et rendre ces
modifications persistantes en les _"committant"_ dans une nouvelle image.

- Quelle est la version la plus récente de l'image `debian` que vous avez trouvé ?
    - Après m'être renseigné, il semblerait que la version la plus récente de
    `debian` serait la 12.5

```bash
docker pull debian:12.5
```

Démarrez alors un container nommé `mydebian` instancié depuis l'image que vous
venez de télécharger.

```bash
docker run --name mydebian -it debian:12.5 /bin/bash
```

- Suppression des répertoires `/opt` `/mnt` et de la commande `/usr/bin/uniq` :

```bash
rm -rf /opt /mnt
rm /usr/bin/uniq
```

- Création du fichier `/home/random` de 8500 bytes à partir de `/dev/urandom` à
l'aide de la commande `dd` :

```bash
dd if=/dev/urandom of=/home/random bs=8500 count=1
```

- Ajout à la fin du fichier `/etc/motd` du message "Tagada tsoin tsoin"

```bash
echo "Tagada tsoin tsoin" >> /etc/motd
```

À l'aide de `docker diff`, affichez les différences entre votre container
`mydebian` et l'image utilisée pour instancié celui-ci.

```bash
docker diff mydebian
```

```bash
C /home
A /home/random
C /etc
C /etc/motd
C /usr
C /usr/bin
D /usr/bin/uniq
D /opt
D /mnt
```

- Quelle est la signification de la lettre située dans la première colonne des
résultats de `docker diff` ?
    - C $\Rightarrow$ _Changed_
    - A $\Rightarrow$ _Added_
    - D $\Rightarrow$ _Deleted_

À l'aide de la commande `docker commit`, committez alors ces changements en créeant
la nouvelle image `debian:new`.

```bash
docker commit mydebian debian:new
```

Détruisez alors le container `debian` précédent puis inspectez l'historique
de l'image `debian:new` que vous venez de créer.

```bash
docker rm mydebian
```

- Combien de couches ont été ajoutées à l'image suite à vos modifications

```bash
docker inspect <image> | jq '.[0].RootFS.Layers'
```

- Image originale :

```bash
[
  "sha256:072686bcd3db19834cd1e0b1e18acf50b7876043f9c38d5308e5e579cbefa6be"
]
```

- Image modifiée :

```bash
[
  "sha256:072686bcd3db19834cd1e0b1e18acf50b7876043f9c38d5308e5e579cbefa6be",
  "sha256:533b8a2c0530e4620b811c83a318b26d690438fe3879dc190cd61bc4a1d89447"
]
```

Nous pouvons par conséquent remarquer qu'une nouvelle couche fut rajoutée dans
la nouvelle image

- Quel est la taille de cette nouvelle couche ?

```bash
docker history debian:new
```

```bash
IMAGE          CREATED       CREATED BY                                      SIZE      COMMENT
51e7bace94f8   4 hours ago   /bin/bash                                       8.8kB
d2a2c1ada45a   2 weeks ago   /bin/sh -c #(nop)  CMD ["bash"]                 0B
<missing>      2 weeks ago   /bin/sh -c #(nop) ADD file:2cc4cba2834c189d0…   117MB
```

Il semblerait donc que la taille de cette nouvelle couche est de 8.8kB

## Exercice 4

On désire enfin se familiariser avec l'utilisation de _volumes_ et _bind mounts_.

Sur votre machine client, créez le volume `pics`, puis montez le dans le repertoire
`/vol/pics` d'un container `ubuntu:22.04` créé pour l'occasion.

### Créez du volume `/pics` sur le client

```bash
docker volume create pics
```

### Montez le volume dans le répertoire dans `/vol/pics` 

```bash
docker run --mount type=volume,src=pics,dst=/vol/pics -it ubuntu:22.04 /bin/bash
```

- Dans le container, quel est le contenu du volume `/pics` ?

Le contenu du volume `/pics` dans le container est vide.

### Copie de données vers le container (volume)

Depuis la machine cliente, copiez un répertoire contenant une série d'images dans
le volume `/pics` et vérifiez que vous pouvez accéder à ces images dans votre
container.

```bash
docker cp . <nom_container>:/vol/pics
```

### Copie des données depuis le container vers la machine locale (volume)

```bash
docker cp <nom_container>:/vol/pics .
```

- Est-ce que les opérations ci-dessus sur le volume `/pics` fonctionnent correctement
si le _daemon_ `dockerd` s'exécute sur une machine différente du client `docker` ?
Justifiez.
    - Les opérations ci-dessus s'exécutent parfaitement suite au fait que les
    volumes `docker` ne nécessitent justement pas d'être stocké sur la même et
    unique machine. Ceci représente un de leur grand avantages par rapport
    aux _bind mounts_.

- Même question mais pour les _bind mounts_.
    - Non, ces opérations ne fonctionneront pas suite au fait que les _bind mounts_
    nécessitent de se "situer" physiquement sur la même machine, ce qui n'est pas
    le cas des _volume mounts_.
