\newpage

# Dockerfiles

## Préréquis

**Vérifier que le paquet `docker-buildx` soit installé sur la machine exécutant
le _daemon_ `dockerd` sinon c'est la hess...**

### Pour les rookies

```bash
sudo apt install docker-buildx
```

### Pour les gens normaux

```bash
sudo pacman -S docker-buildx
```

## Exercice 1

### Création d'une image à partir du Dockerfile

```bash
docker buildx build . -t dockerfiles_ex01:1.0
```

### Contenu du Dockerfile

```bash
FROM alpine:latest
LABEL maintainer="Yoda"
ADD files.tar.gz /dir_tar
RUN mkdir /dir_add
COPY file0* /dir_add/
CMD ["ls", "-al", "/dir_tar", "/dir_add"]
```

- `FROM` : spécifie l'image parent de notre nouvelle image, en l'occurrence
le parent sera `alpine:latest`.

- `LABEL` : cette commande ajoute des métadonnées concernant l'image. Dans notre
cas, on lui spécifie la paire `<clé><valeur>` $\rightarrow$ `maintainer:"Yoda"`.

- `ADD` : on spécifie qu'on souhaite non seulement copier (la commande `COPY`
suffirait) la _tarball_ `files.tar.gz` mais aussi de l'extraire suite au fait
que `ADD` possède des fonctionnalités supplémentaires par rapport à `COPY`.
Cette _tarball_ sera extraite dans le répertoire `/dir_tar` du container.

- `RUN` : indique que l'on souhaite exécuter une commande, donc le répertoire
on va créer le répertoire `/dir_add`.

- `COPY` : va simplement copier les fichiers locaux commençant par `files0` dans
le répertoire `/dir_add` du container.

- `CMD` : spécifie la commande (avec ses arguments) qui sera lancée au moment
de l'instantiation du container. Dans notre cas, au lancement du container
le contenu des répertoire `/dir_tar` et `/dir_add` seront affichés sous forme
de liste (`-l`). Les fichier cachés seront aussi affichés grâce au flag (`-a`).


### Pourquoi le fichier `file04` n'est pas présent dans le répertoire `/dir_add` ?

Pour voir pourquoi `file04` n'est pas présent dans le répertoire `/dir_add`, il suffit d'ouvrir le dossier contenant le `Dockerfile` et de s'apercevoir qu'il y a un fichier `.dockerignore` qui spécifie que le fichier `file04` ne doit pas être copié dans le container. Il est ajouté dans le dossier `/dir_tar` suite à l'exécution de la commande `ADD` mais n'est pas copié dans le dossier `/dir_add` suite à l'exécution de la commande `COPY`.


### Observez les attributs des fichiers ajoutés à l'exécution du container. D'où vient cette différence ?

```bash
docker run dockerfiles_ex01:1.0
```

```bash
/dir_add:
total 32
drwxr-xr-x    1 root     root          4096 May 31 22:02 .
drwxr-xr-x    1 root     root          4096 May 31 22:39 ..
-rw-rw-r--    1 root     root            36 May 27 08:38 file01
-rw-rw-r--    1 root     root          2796 May 27 08:38 file02
-rw-rw-r--    1 root     root         13990 May 27 08:38 file03

/dir_tar:
total 108
drwxr-xr-x    2 root     root          4096 May 31 22:02 .
drwxr-xr-x    1 root     root          4096 May 31 22:39 ..
-rw-rw-r--    1 1000     1000            36 Apr 20  2023 file01
-rw-rw-r--    1 1000     1000          2796 Apr 20  2023 file02
-rw-rw-r--    1 1000     1000         13990 Apr 20  2023 file03
-rw-rw-r--    1 1000     1000         75133 Apr 20  2023 file04
```

Mon hypothèse est que lors de l'extraction de la _tarball_, les fichiers contenu
dans l'archive conservent les permissions / propriétaires / groupes associés
au moment de leur archivage. Le valeur `1000` pour le propriétaire / groupe
des fichiers contenu dans `/dir_tar` correspond à la valeur par défaut associée
à un compte utilisateur sur `GNU/Linux`. Quant aux fichiers contenu dans le
répertoire `/dir_add`, la raison pour laquelle eux appartiennet à `root` est
simplement dû au fait qu'à l'exécution du container, on est par défaut `root`.
Par conséquent, lorsque le `Dockerfile` a exécuté la commande `COPY` en étant
`root` à ce moment-là, alors les valeurs propriétaire / groupe des fichiers
copiés dans le nouveau répertoire ont été spécifié en tant que `root`.

> Once again, not sure..

### Inspectez l'historique (non-tronquée) de l'image et décrivez chacune des ses couches

```bash
docker history dockerfiles_ex01:1.0 --no-trunc
```

```bash
IMAGE                                                                     CREATED          CREATED BY                                                                                          SIZE
sha256:969768d594b45fd014a90986623fa0ac9ede6ee8d26f0e4dd9532f93f9c63c2b   50 minutes ago   CMD ["ls" "-al" "/dir_tar" "/dir_add"]                                                              0B
<missing>                                                                 50 minutes ago   COPY file0* /dir_add/ # buildkit                                                                    16.8kB
<missing>                                                                 50 minutes ago   RUN /bin/sh -c mkdir /dir_add # buildkit                                                            0B
<missing>                                                                 50 minutes ago   ADD files.tar.gz /dir_tar # buildkit                                                                92kB
<missing>                                                                 50 minutes ago   LABEL maintainer=Yoda                                                                               0B
<missing>                                                                 9 days ago       /bin/sh -c #(nop)  CMD ["/bin/sh"]                                                                  0B
<missing>                                                                 9 days ago       /bin/sh -c #(nop) ADD file:e3abcdba177145039cfef1ad882f9f81a612a24c9f044b19f713b95454d2e3f6 in /    7.79MB
```

#### Explication des diverses couches

On va tenter d'expliquer les couches qui composent cette image à partir de
la plus ancienne vers la plus récente.

- Les deux premières couches (qui ont été créé il y a 9 jours de cela),
correspondent aux couches de l'image parent `alpine:latest` (commande `FROM`
dans le `Dockerfile`).

- Par la suite, chaque nouvelle ligne dans le `Dockerfile` engendre une
nouvelle couche, raison pour laquelle une couche est associée aux commandes
`LABEL`, `ADD`, `RUN`, `COPY` ainsi que `CMD`.

- Dans le cas de la dernière commande `CMD`, le _digest_ SHA-256 de cette
est affichée.

> Je ne suis pas certain de la raison pour laquelle je ne vois pas ce _digest_
pour les autres couches créées par le `Dockerfile`, à part dans le cas des deux
plus anciennes car elles sont associées à l'image parent et il semblerait
qu'il ne soit pas possible de remonter dans l'historique de l'image parent.

### Création de l'image en tant que `root`. Cela change-t-il quelque chose ?

Cela semblerait ne changer strictement rien, possiblement suite au fait que
l'on soit déjà par défaut `root` dans le container, par conséquent la création
de l'image en étant `root` ne change rien quant aux attributs de l'image et
le comportement du container instancié à partir de celle-ci.

> prolly, bs ngl

## Exercice 2

### Contenu du `Dockerfile.convert`

```bash
FROM archlinux:base-devel-20240101.0.204074

RUN pacman -Sy && pacman -S imagemagick --noconfirm

WORKDIR /data

ENTRYPOINT ["mogrify"]
CMD ["-help"]
```

### Commande permetant la fabrication de l'image à partir du Dockerfile

```bash
docker buildx build . -t imgconvert:1.0 -f Dockerfile.convert
```

### Script `mogrifydock.sh`  permettant l'utilisation de l'image

```bash
#!/usr/bin/env bash

if [ $# -eq 0 ]; then
        docker run --rm \
                imgconvert:1.0
        exit 0
fi

if [ $# -ne 2 ]; then
        echo "Usage: $0 \"<new_format>\" \"<images_to_convert>\""
        exit 1
fi

format=$1
images=$2

docker run --rm \
        --volume "$(pwd):/data" \
        --user $(id -u):$(id -g) \
        imgconvert:1.0 -format $format $images
```

### Exemple d'utilisation 

Afin de pouvoir convertir toutes les photos `.jpg` du répertoire courant dont
le nom débute par la lettre "a", il est nécessaire d'exécuter le script
`mogrifydock.sh` de la manière suivante :

```bash
./mogrifydock.sh "png" "a*.jpg"
```

## Exercice 3

### Partie 1

```bash
FROM debian:12-slim

RUN apt update -y && apt install -y wget tar make gcc libncurses-dev && \
        wget https://www.alessandropira.org/alienwave/alienwave-0.4.0.tar.gz && \
        tar -xzf alienwave-0.4.0.tar.gz alienwave && cd /alienwave/ && make && \
        apt remove -y --purge wget make gcc libncurses-dev

WORKDIR /alienwave

ENTRYPOINT ["./alienwave"]
```

#### Taille de l'image

```bash
REPOSITORY   TAG       IMAGE ID       CREATED          SIZE
alienwave    1.0       f4ee2cb8bbb4   46 seconds ago   340MB
```

### Partie 2

- Quel est le nom de la dernière image `alpine` ?
    - `alpine:3.20` 


#### Dockerfile

```bash
FROM ubuntu:22.04 as BUILDER

RUN apt update -y && apt install libncurses-dev wget -y
RUN apt install -y build-essential
RUN wget https://www.alessandropira.org/alienwave/alienwave-0.4.0.tar.gz
RUN tar -xzf alienwave-0.4.0.tar.gz
RUN cd alienwave/ && sed -i 's/LIB = -lncurses/LIB = -lncurses -ltinfo -static/g' Makefile && make

FROM alpine:3.20
RUN apk update && apk add ncurses
COPY --from=builder /alienwave/alienwave /usr/bin

ENTRYPOINT ["/usr/bin/alienwave"]
```

- On a réussi à atteindre une taille de 12MB.

```bash
REPOSITORY             TAG       IMAGE ID       CREATED         SIZE
alienwave_multistage   1.0       1c7f0a242c2c   4 minutes ago   12MB
```

## Exercice 4

### Partie 1

#### Dockerfile

```bash
FROM ubuntu:22.04

RUN apt update -y && apt install -y golang ca-certificates && go install github.com/alfg/asciicat@latest

WORKDIR /root

ENV PATH="${PATH}:/root/go/bin"

WORKDIR /home/Pictures

COPY . .

ENTRYPOINT ["asciicat"]
```

#### Fabrication de l'image

```bash
docker buildx build . -t asciicat:1.0 -f Dockerfile1
```

#### Utilisation

```bash
docker run --rm asciicat:1.0 -i tux.png
```

- Quelle est la taille de l'image générée ?

```bash
docker run --rm asciicat:1.0 --help
```

```bash
Usage of usr/bin/asciicat:
  -i string
        Use -i <filename> (default "gopher.png")
  -w int
        Use -w <width> (default 80)
```

Donc la largeur par défaut de l'image est 80

- Quelle est la ligne de commande à exécuter pour obtenir une largeur de 40

```bash
docker run --rm asciicat:1.0 -i cat.png -w 40
```

### Partie 2

#### Dockerfile

```bash
FROM ubuntu:22.04 as BUILDER

RUN apt update -y && apt install -y golang ca-certificates
RUN go install github.com/alfg/asciicat@latest


FROM alpine:3.20

RUN apk update
COPY --from=builder /root/go/bin/asciicat /usr/bin
COPY . .

ENTRYPOINT ["usr/bin/asciicat"]
```

#### Affichez la différence de taille entre l'image initiale et la _multi-stage build_

```bash
REPOSITORY            TAG       IMAGE ID       CREATED          SIZE
asciicat_multistage   1.0       a10aafdcc5c2   12 seconds ago   12.8MB
asciicat              1.0       5f22c52d66b2   7 minutes ago    890MB
```

- Le facteur de taille gagnée est de $\sim 69$ (noice) fois.

### Partie 3

### Création d'archives locales pour `ubuntu:22.04` et `alpine:3.20`

En premier lieu, afin de pouvoir s'assurer que nous nous baserons sur des images
"figées" dans le temps, nous devons d'abord exporter le `rootfs` locaux en
instanciant des containers à partir des images `ubuntu:22.04` et `alpine:3.20`.

```bash
docker run --rm -it ubuntu:22.04 /bin/sh
```

Nous instancions le(s) container(s) en mode intéractif de sorte à pouvoir se
détacher sans qu'il ne se termine immédiatement en exécutant `Ctrl-p` et
`Ctrl-q`.

À présent, nous pouvons créer les archives en exécutant la commande suivante :

```bash
docker export <nom_container> > <nom_archive>.tar
```

Le nom du container peut être retrouvé en listant les containers actifs avec la
commande `docker ps`.

### Création de l'image `asciicat` à partir des archives locales

À présent, nous pouvons utiliser ces archives pour notre image `asciicat`.

```bash
FROM scratch as BUILDER
ADD ubuntu_22.04.tar /

RUN apt update -y && apt install -y golang ca-certificates
RUN go install github.com/alfg/asciicat@latest


FROM scratch
ADD alpine_3.20.tar /

RUN apk update
COPY --from=builder /root/go/bin/asciicat /usr/bin
COPY . .

ENTRYPOINT ["usr/bin/asciicat"]
```
