\newpage

# QEMU avancé

## Conversion d'images disque

### Quels sont les formats d'images disponibles et leurs extensions respectives ?

Deux types d'images de la distribution Linux Debian 11 Bullseye **server** (64-bit)
sont disponibles :
- VirtualBox $\Rightarrow$ `.vdi` 
- VMware $\Rightarrow$ `.vmdk` 

> _"Un hyperviseur de Type 2 est un logiciel qui s'exécute à l'intérieur d'un 
autre système d'exploitation"_
>
> -- <cite>[Hyperviseur](https://fr.wikipedia.org/wiki/Hyperviseur)</cite>

### Exécutez ensuite une VM QEMU sur chaque format d'image téléchargé

Initialement, il n'est pas possible de le faire car les formats `.vdi` et `.vmdk`
ne sont pas géré comme des images natives au format `qcow2`. Par conséquent,
il est nécessaire de le convertir dans le bon format grâce à l'outil `qemu-img
convert`.

```bash
qemu-img convert -O qcow2 <img_src> <img_qcow2>
```

### Modification de la taille du cluster d'une image

L'exemple ci-dessous montre comment spécifier une taille de cluster de 2MB.

```bash
qemu-img convert -O qcow2 -o cluster_size=2M <img_src> <img_qcow2>
```

## Inspection et hacking d'image disque

### Préparation de l'image

#### Téléchargement de l'image

```bash
curl -o vm2fix.qcow.gz https://drive.switch.ch/index.php/s/L8RFwPeeJcrVN8m/download 
```

#### Décompression du fichier

```bash
gzip -d vm2fix.qcow.gz
```

### Combien de partitions comporte `vm2fix.qcow` et de quels types sont-elles ?

L'image contient 3 partitions :

```bash
virt-filesystems -a vm2fix.qcow -l
```

```bash
Name       Type        VFS   Label  Size          Parent
/dev/sda1  filesystem  ext4  -      232042008576  -
/dev/sda2  filesystem  ext4  -      965390336     -
/dev/sda4  filesystem  ext4  -      284281606144  -
```

### Que contient chaque partition ?

#### `/dev/sda1` 

Cette partition contient le système de fichiers racine `/`.

#### `/dev/sda2` 

Cette partition contient le bootloader `GRUB`.

#### `/dev/sda4`

Cette partition contient les dossiers `/home` des divers utilisateurs.


### Quel(s) utilisateur(s) "normal" $(\text{UID} \geqslant 1000)$ avez-vous déterminé ?

```bash
fabricebellard:x:1000:1000:Fabrice Bellard,,,:/home/fabricebellard:/bin/bash
linustorvalds:x:1001:1001:Linus Torvalds,,,:/home/linustorvalds:/bin/bash
```

### Modification pour se connecter en tant que `root` et en tant qu'utilisateur normal

Pour cela, il est nécessaire de modifier le fichier `/etc/passwd` en retirant
le caractère `x` à la suite du nom d'utilisateur comme ci-dessous :

```bash
root::0:0:root:/root:/bin/bash
fabricebellard::1000:1000:Fabrice Bellard,,,:/home/fabricebellard:/bin/bash
linustorvalds::1001:1001:Linus Torvalds,,,:/home/linustorvalds:/bin/bash
```

## Snapshots

### Démarrez deux VMs à partir d'une seule et même image disque. Que remarquez-vous ? Quel est la raison de ce comportement ?

Il n'est pas possible d'exécuter deux ou plus VMs à partir d'une seule et même
image disque, car le processus initial (la 1ère VM créée) doit avoir un accès 
"atomique" au disque, raison pour laquelle le message d'erreur nous dit que le nouveau
processus n'a pu obtenir le verrou afin d'obtenir un accès en écriture.

```bash
qemu-system-x86_64: disk_base.qcow: Failed to get "write" lock
Is another process using the image [disk_base.qcow]?
```

### Schéma des snapshots

#### Étape 1

```bash
qemu-img create -F qcow2 -b disk_base.qcow -f qcow2 snap1.qcow
```

#### Étape 2

Création du fichier `fileA` avec le script `createfile`  en exécutant la VM à
partir de `snap1`.

```bash
vm_run -d snap1
```

Dans la VM :

```bash
./createfile.sh fileA
```

#### Étape 3

```bash
qemu-img create -F qcow2 -b snap1.qcow -f qcow2 snap2.qcow
```

#### Étape 4

```bash
qemu-img create -F qcow2 -b snap2.qcow -f qcow2 snap3.qcow
```

#### Étape 5

Création du fichier `fileB` avec le script `createfile`  en exécutant la VM à
partir de `snap3`.

```bash
vm_run -d snap3
```

Dans la VM :

```bash
./createfile.sh fileB
```

#### Étape 6

```bash
qemu-img create -F qcow2 -b snap2.qcow -f qcow2 snap4.qcow
```

#### Étape 7

Création du fichier `fileC` avec le script `createfile`  en exécutant la VM à
partir de `snap4`.

```bash
vm_run -d snap4
```

Dans la VM :

```bash
./createfile.sh fileC
```

### Vous exécutez une VM sur l'image `snap3`. Quel(s) fichier(s) seront présent(s) ? 

Sur cette image, les fichier `fileA` et `fileB` sont présents.

### Vous exécutez une VM sur l'image `snap4`. Quel(s) fichier(s) seront présent(s) ?

Sur cette image, les fichier `fileA` et `fileC` sont présents.

### Chaîne d'images ayant mené à `snap3` 

```bash
qemu-img info --backing-chain snap3
```

```bash
image: snap3.qcow
file format: qcow2
virtual size: 100 GiB (107374182400 bytes)
disk size: 132 MiB
cluster_size: 65536
backing file: snap2.qcow
backing file format: qcow2
Format specific information:
    compat: 1.1
    compression type: zlib
    lazy refcounts: false
    refcount bits: 16
    corrupt: false
    extended l2: false
Child node '/file':
    filename: snap3.qcow
    protocol type: file
    file length: 132 MiB (138608640 bytes)
    disk size: 132 MiB

image: snap2.qcow
file format: qcow2
virtual size: 100 GiB (107374182400 bytes)
disk size: 196 KiB
cluster_size: 65536
backing file: snap1.qcow
backing file format: qcow2
Format specific information:
    compat: 1.1
    compression type: zlib
    lazy refcounts: false
    refcount bits: 16
    corrupt: false
    extended l2: false
Child node '/file':
    filename: snap2.qcow
    protocol type: file
    file length: 194 KiB (198656 bytes)
    disk size: 196 KiB

image: snap1.qcow
file format: qcow2
virtual size: 100 GiB (107374182400 bytes)
disk size: 123 MiB
cluster_size: 65536
backing file: disk_base.qcow
backing file format: qcow2
Format specific information:
    compat: 1.1
    compression type: zlib
    lazy refcounts: false
    refcount bits: 16
    corrupt: false
    extended l2: false
Child node '/file':
    filename: snap1.qcow
    protocol type: file
    file length: 123 MiB (128647168 bytes)
    disk size: 123 MiB

image: disk_base.qcow
file format: qcow2
virtual size: 100 GiB (107374182400 bytes)
disk size: 12.4 GiB
cluster_size: 65536
Format specific information:
    compat: 1.1
    compression type: zlib
    lazy refcounts: false
    refcount bits: 16
    corrupt: false
    extended l2: false
Child node '/file':
    filename: disk_base.qcow
    protocol type: file
    file length: 12.4 GiB (13269532672 bytes)
    disk size: 12.4 GiB
```

### Chaîne d'images ayant mené à `snap4` 

```bash
qemu-img info --backing-chain snap4
```

```bash
image: snap4.qcow
file format: qcow2
virtual size: 100 GiB (107374182400 bytes)
disk size: 123 MiB
cluster_size: 65536
backing file: snap2.qcow
backing file format: qcow2
Format specific information:
    compat: 1.1
    compression type: zlib
    lazy refcounts: false
    refcount bits: 16
    corrupt: false
    extended l2: false
Child node '/file':
    filename: snap4.qcow
    protocol type: file
    file length: 123 MiB (129368064 bytes)
    disk size: 123 MiB

image: snap2.qcow
file format: qcow2
virtual size: 100 GiB (107374182400 bytes)
disk size: 196 KiB
cluster_size: 65536
backing file: snap1.qcow
backing file format: qcow2
Format specific information:
    compat: 1.1
    compression type: zlib
    lazy refcounts: false
    refcount bits: 16
    corrupt: false
    extended l2: false
Child node '/file':
    filename: snap2.qcow
    protocol type: file
    file length: 194 KiB (198656 bytes)
    disk size: 196 KiB

image: snap1.qcow
file format: qcow2
virtual size: 100 GiB (107374182400 bytes)
disk size: 123 MiB
cluster_size: 65536
backing file: disk_base.qcow
backing file format: qcow2
Format specific information:
    compat: 1.1
    compression type: zlib
    lazy refcounts: false
    refcount bits: 16
    corrupt: false
    extended l2: false
Child node '/file':
    filename: snap1.qcow
    protocol type: file
    file length: 123 MiB (128647168 bytes)
    disk size: 123 MiB

image: disk_base.qcow
file format: qcow2
virtual size: 100 GiB (107374182400 bytes)
disk size: 12.4 GiB
cluster_size: 65536
Format specific information:
    compat: 1.1
    compression type: zlib
    lazy refcounts: false
    refcount bits: 16
    corrupt: false
    extended l2: false
Child node '/file':
    filename: disk_base.qcow
    protocol type: file
    file length: 12.4 GiB (13269532672 bytes)
    disk size: 12.4 GiB
```

### Exécutez une VM à partir de `snap3` puis une autre à partir de `snap1`. Que se passe-t-il ?

Il est impossible d'exécuter une VM à partir de `snap1` en ayant lancé au préalable
une VM à partir de `snap3` car vu que le snapshot `snap3` dépend de `snap1`,
il aura besoin d'avoir un accès exclusif (un verrou) sur le snapshot `snap1` afin
de pouvoir s'exécuter.

### Effectuer un _merge_ de type "commit" de l'image `snap4` dans `snap1` 

```bash
qemu-img commit -f qcow2 -b <dst> <src>
```

```bash
qemu-img commit -f qcow2 -b snap1.qcow snap4.qcow
```

### Quels fichiers seront présents dans `snap1`

Après le merge de `snap4` dans `snap1`, les fichiers `fileA` et `fileC` seront
présents dès lors dans `snap1`.

### Quelle(s) image(s) pourrai(en)t être incohérente(s) ?

Vu que l'image `snap4` se basait sur les images `snap1` et `snap2`, par conséquent,
après avoir _merge_ `snap4` dans `snap1`, cela invaliderait `snap4` et `snap2`.
On peut effacer sans aucun soucis `snap4`, cependant en ce qui concerne `snap2`
on ne peut pas le faire car `snap3` dépend d'elle.

### Qu'auriez-vous pu faire pour éviter de réaliser cette copie de plusieurs GB tout en évitant de modifier le fichier d'origine ?

On aurait pu effectuer des snapshots internes directement dans l'image originale
`lubuntu.qcow`.

### Réalisation du snapshot en cours de jeu

```bash
savevm <tag>
```

### Pourquoi est-ce que la création de ce snapshot (interne) prend significativement plus de temps qu'un snapshot de disque ?

Ceci est dû au fait qu'un snapshot interne conserve non seulement l'état du
disque mais aussi de la mémoire (RAM) ainsi que l'état général de la VM.

### Listez les snapshots

```bash
info snapshots
```

### Chargez un snapshot

```bash
loadvm <tag>

```

### Listez les snapshots en dehors de QEMU _monitor_ ?

```bash
qemu-img snapshot -l snap1.qcow 
```

### Exécutez à nouveau la VM à partir d'un snapshot à l'aide de l'option `loadvm` mais avec des périphériques différents. Que se passe-t-il ?

Il est impossible de recharger la VM à partir d'un snapshot possédant une
autre configuration matérielle. Par conséquent, un message d'erreur apparaît.

```bash
qemu-system-x86_64: Unknown savevm section or instance 'cpu_common' 2. Make sure that your current VM setup matches your saved VM setup, including any hotplugged devices
qemu-system-x86_64: Error -22 while loading VM state
```

## Virtualisation de _desktop_ (VDI)

### Téléchargement de l'image `doom.qcow`

La première chose à faire est de télécharger l'image compressée en local, pas
sur le serveur distant, puis de l'extraire

```bash
curl -o doom.qcow.gz https://drive.switch.ch/index.php/s/rHScyU6IVNyoZYD/download
```

```bash
gzip -d doom.qcow.gz
```

### Conversion de l'image en _sparse file_

Afin de récupérer de l'espace non-utilisée dans l'image decompressée de 15GB,
il est possible de la convertir en _sparse-file_ à l'aide de la commande ci-dessous :

```bash
fallocate -v -d doom.qcow
```

```bash
doom.qcow: 4.3 GiB (4637921280 bytes) converted to sparse holes
```

Ceci nous a permis d'économiser un peu plus de 4GB. À présent il est possible
de transférer l'image sur le serveur distant.

### Transfert de l'image `doom.qcow` sur le serveur distant

Vu que `scp` **ne supporte pas** les fichiers _sparse_, il est nécessaire
d'effectuer le transfert grâce à `rsync` à travers le réseau.

```bash
rsync -zP --sparse doom.qcow <username>@<server_addr>:<dst_path_server>
```

### Modification du script `vm_run.sh` de sorte à pouvoir lancer un serveur `spice`

```bash
MEM_COUNT="4G"
CPU_COUNT="2"
NET_PV="-nic user,model=virtio-net-pci"
GPU_PV="-vga virtio"

SPICE_SERVER="-device virtio-serial-pci \
                -spice port=8000,disable-ticketing=on \
                -device virtserialport,chardev=spicechannel0,name=com.redhat.spice.0 \
                -chardev spicevmc,id=spicechannel0,name=vdagent,clipboard=on"

QEMU_GA="-device virtio-serial \
            -device virtserialport,chardev=qga0,name=org.qemu.guest_agent.0 \
            -chardev socket,path=/tmp/qga.sock,server=on,wait=off,id=qga0"

if $DISK_OPT; then
    qemu-system-x86_64 -m $MEM_COUNT -smp cpus=$CPU_COUNT $NET_PV \
        $GPU_PV $DISK_PATH $QEMU_GA $SPICE_SERVER
fi
```

### Lancement du client sur la machine local

```bash
remote-viewer "spice://10.136.27.45?port=8000"
```

### Le jeu est-il jouable ?

Oui.

### Même question mais depuis chez vous avec le VPN ?

Oui.

### Sur quelle machine physique s'exécute le jeu exactement ?

Sur le serveur distant.

### Vous devriez entrendre la bande sonore du jeu pourtant le serveur ne possède pas de carte audio ?

La carte audio est émulée.

### Si vous fermez la fenêtre du client, que se passe-t-il du côté du serveur ?

La VM continue de fonctionner même si le client s'est déconnecté.

### Vous est-il possible de copier coller du texte entre la VM et la machine hôte

Oui, magie.
